import re

from jirafs import constants, exceptions
from jirafs.readers import GitRevisionReader, WorkingCopyReader


class JiraLinkManager(dict):
    TICKET_MATCHER = re.compile(r"[A-Za-z]+-\d+")

    def __init__(self, data, prepared=False):
        if not prepared:
            self._data = self.get_fields_from_string(data)
        else:
            self._data = data
        super(JiraLinkManager, self).__init__(self._data)

    @classmethod
    def create(cls, folder, revision=None, path=None):
        if revision and path:
            raise TypeError("You may specify a git revision or a local path; not both.")

        if revision:
            return GitRevisionJiraLinkManager(folder, revision)
        else:
            return WorkingCopyJiraLinkManager(folder, path)

    def __sub__(self, other):
        differing = {}

        slf = {}
        for category in ["remote", "issue"]:
            slf[category] = self[category].copy()
            for k, v in other[category].items():
                if k not in slf[category]:
                    if category not in differing:
                        differing[category] = {}
                    differing[category][k] = (
                        v,
                        None,
                    )
                    continue
                l_v = slf[category].pop(k)
                if l_v != v:
                    if category not in differing:
                        differing[category] = {}
                    differing[category][k] = (
                        v,
                        l_v,
                    )

            for k, v in slf[category].items():
                if category not in differing:
                    differing[category] = {}
                differing[category][k] = (None, v)

        return differing

    def get_links_from_string(self, contents):
        links = {
            "remote": {},
            "issue": {},
        }

        for line in contents.split("\n"):
            if not line.startswith("*"):
                continue
            line = line[1:].strip()
            try:
                left, right = line.split(": ", 1)
            except ValueError:
                raise exceptions.IssueValidationError(
                    "Remote links must have a description; format your "
                    "link to {url} like '* Your Title: {url}' "
                    "to continue.".format(url=line.strip())
                )
            left = left.strip()
            right = right.strip()
            if self.TICKET_MATCHER.match(right):
                right = right.upper()
                links["issue"][right] = {"status": left.lower()}
            else:
                links["remote"][right] = {}
                if left:
                    links["remote"][right]["description"] = left

        return links


class AutomaticJiraLinkManager(JiraLinkManager):
    def __init__(self):
        data = self.load()
        super(AutomaticJiraLinkManager, self).__init__(data, prepared=True)

    def load(self):
        return self.get_links_from_string(
            self.get_file_contents(constants.TICKET_LINKS)
        )

    def get_file_contents(self, path):
        raise NotImplementedError()


class GitRevisionJiraLinkManager(GitRevisionReader, AutomaticJiraLinkManager):
    pass


class WorkingCopyJiraLinkManager(WorkingCopyReader, AutomaticJiraLinkManager):
    pass



from __future__ import print_function

import json

from prettytable import PrettyTable

from jirafs import utils
from jirafs.plugin import CommandResult, DirectOutputCommandPlugin
from jirafs.ticketfolder import TicketFolder


class Command(DirectOutputCommandPlugin):
    """Search for users matching the specified search term"""

    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"
    AUTOMATICALLY_INSTANTIATE_FOLDER = False

    def main(self, args, jira, path, parser, **kwargs):
        search_term = " ".join(args.terms)

        try:
            tf = TicketFolder(path, jira)
            jira_client = tf.jira
        except IOError:
            jira_client = jira(utils.get_default_jira_server())

        users = jira_client.search_users(search_term)

        if args.json:
            print(json.dumps([u.raw for u in users]))
            return

        if not users:
            return CommandResult("No matching users were found", return_code=1)

        table = PrettyTable(["Name", "Username", "Email Address", "Time Zone"])
        table.align = "l"
        for user in users:
            table.add_row(
                [user.displayName, user.name, user.emailAddress, user.timeZone]
            )

        return CommandResult(str(table))

    def add_arguments(self, parser):
        parser.add_argument("terms", nargs="*")
        parser.add_argument("--json", default=False, action="store_true")



from environmental_override import override  # noqa

from jirafs import __version__ as version

# Metadata filenames
TICKET_DETAILS = "fields.jira"
TICKET_COMMENTS = "comments.read_only.jira"
TICKET_NEW_COMMENT = "new_comment.jira"
TICKET_LINKS = "links.jira"
TICKET_FILE_FIELD_TEMPLATE = "{field_name}.jira"

# Generic settings
LOCAL_ONLY_FILE = ".jirafs_local"
REMOTE_IGNORE_FILE = ".jirafs_remote_ignore"
GIT_IGNORE_FILE_PARTIAL = ".jirafs_ignore"
GIT_IGNORE_FILE = ".jirafs/combined_ignore"
GIT_EXCLUDE_FILE = ".jirafs/git/info/exclude"
TICKET_OPERATION_LOG = "operation.log"
METADATA_DIR = ".jirafs"
GLOBAL_CONFIG = ".jirafs_config"
TEMP_GENERATED_FILES = ".jirafs/temp-generated"
GIT_AUTHOR = "Jirafs %s <jirafs@localhost>" % (version)
DEFAULT_BRANCH = "master"

# Config sections
CONFIG_JIRA = "jira"
CONFIG_MAIN = "main"
CONFIG_PLUGINS = "plugins"

NO_DETAIL_FIELDS = ["comment", "watches", "attachment"]
FILE_FIELDS = [
    "description",
]
FILE_FIELD_BLACKLIST = [
    "new_comment",
    "fields",
    "links",
]

ALLOW_USER_INPUT = True

DEFAULT_DATE_FORMAT = "%Y-%m-%d at %H:%M:%S %Z"

CURRENT_REPO_VERSION = 17


override(locals(), "JIRAFS_")



from jirafs import utils
from jirafs.plugin import CommandPlugin


class Command(CommandPlugin):
    """Create a subtask of a given issue."""

    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def main(self, folder, args, **kwargs):
        summary = " ".join(args.summary)

        issue_data = {
            "project": {"key": folder.issue.fields.project.key},
            "summary": summary,
            "issuetype": {"name": "Sub-task"},
            "parent": {"id": folder.issue.key},
        }

        if args.issuetype is not None:
            issue_data["issuetype"]["name"] = args.issuetype

        folder.jira.create_issue(fields=issue_data)

        commands = utils.get_installed_commands()
        jira = utils.lazy_get_jira()
        commands["fetch"].execute_command(
            [],
            jira=jira,
            path=folder.path,
            command_name="fetch",
        )

    def add_arguments(self, parser):
        parser.add_argument("summary", nargs="+")
        parser.add_argument("--issuetype", default=None, type=str)



import pydoc

from jirafs.plugin import CommandPlugin


class Command(CommandPlugin):
    """Run a git command against this ticketfolder's underlying GIT repo"""

    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def handle(self, args, folder, **kwargs):
        return self.cmd(folder, *self.git_arguments)

    def parse_arguments(self, parser, extra_args):
        args, git_arguments = parser.parse_known_args(extra_args)
        self.git_arguments = git_arguments
        return args

    def main(self, folder, *git_arguments):
        result = folder.run_git_command(*git_arguments)
        pydoc.pager(result)
        return result



from jirafs.plugin import CommandPlugin


class Command(CommandPlugin):
    """Assign the current task to a user"""

    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def main(self, args, folder, **kwargs):
        username = args.username
        if not username:
            username = folder.get_config().get(
                folder.jira_base,
                "username",
            )
        folder.jira.assign_issue(folder.issue, username)
        folder.log(
            "Successfully assigned %s to %s.",
            args=(folder.issue.key, username),
        )

    def add_arguments(self, parser):
        parser.add_argument("username", nargs="?", type=str)



import json
import os
import subprocess

from jirafs.plugin import CommandPlugin, CommandResult


class Command(CommandPlugin):
    """Check whether a given dotpath matches an expected value"""

    TRY_SUBFOLDERS = True
    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def handle(self, args, folder, **kwargs):
        return self.cmd(
            folder,
            args.field_name,
            args.field_value,
            isjson=args.json,
            negate=args.negate,
            raw=args.raw,
            quiet=args.quiet,
            execute=args.execute,
            execute_here=args.execute_here,
        )

    def add_arguments(self, parser):
        parser.add_argument(
            "field_name",
        )
        parser.add_argument(
            "field_value",
        )
        parser.add_argument(
            "--json",
            help=("Process the provided field value as JSON"),
            action="store_true",
            default=False,
        )
        parser.add_argument(
            "--execute",
            help=(
                "Execute a command for each matching result; by default,"
                " will be executed from within the matching folder directory"
                " when executed on a folder containing multiple ticket"
                " folders.  See --execute-here to change this behavior."
                " The string {} will be replaced with the folder directory "
                " path."
            ),
            default=None,
        )
        parser.add_argument(
            "--execute-here",
            help=(
                "Do not switch directories to matching folders's paths"
                " when using --execute."
            ),
            action="store_true",
            default=False,
        )
        parser.add_argument(
            "--negate",
            help=("Compare the field value without applying " "plugin transformations"),
            action="store_true",
            default=False,
        )
        parser.add_argument(
            "--raw",
            help=("Return the field value without applying " "plugin transformations"),
            action="store_true",
            default=False,
        )
        parser.add_argument(
            "--quiet",
            help=("Print no message to stdout indicating success or failure"),
            action="store_true",
            default=False,
        )

    def main(
        self,
        folder,
        field_name,
        field_value,
        isjson,
        negate,
        raw,
        quiet,
        execute,
        execute_here,
    ):
        actual_value = folder.get_field_value_by_dotpath(field_name, raw=raw)

        if isjson:
            field_value = json.loads(field_value)

        success = actual_value == field_value

        comparison_result = " != "
        if success:
            comparison_result = " == "
        message = "{left} {comparison} {right}".format(
            left=actual_value,
            comparison=comparison_result,
            right=field_value,
        )

        if negate:
            success = not success

        if execute and success:
            execute = execute.replace("{}", folder.path)
            subprocess.call(
                execute, shell=True, cwd=os.getcwd() if execute_here else folder.path
            )

        return (
            message if not quiet else None,
            0 if success else 1,
        )

    def cmd(self, *args, **kwargs):
        message, return_code = self.main(*args, **kwargs)

        return CommandResult(message, return_code=return_code)



import json

from jirafs.exceptions import JirafsError
from jirafs.plugin import CommandPlugin, CommandResult


class Command(CommandPlugin):
    """Get the status of the current ticketfolder"""

    TRY_SUBFOLDERS = True
    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def handle(self, args, folder, **kwargs):
        return self.cmd(folder, args.field_name, raw=args.raw, formatted=args.formatted)

    def add_arguments(self, parser):
        parser.add_argument(
            "--raw",
            help=("Return the field value without applying " "plugin transformations"),
            action="store_true",
            default=False,
        )
        parser.add_argument(
            "--formatted",
            help=("Format JSON output with indentation and sorted keys."),
            action="store_true",
            default=False,
        )
        parser.add_argument(
            "field_name",
        )

    def main(self, folder, field_name, raw=False, formatted=False):
        special_fields = {
            "new_comment": folder.get_new_comment,
            "links": folder.get_links,
            "fields": folder.get_fields,
        }
        if field_name in special_fields:
            data = special_fields[field_name]()
        else:
            data = folder.get_field_value_by_dotpath(field_name, raw)

        if isinstance(data, (list, dict)):
            kwargs = {}
            if formatted:
                kwargs = {
                    "indent": 4,
                    "sort_keys": True,
                }
            data = json.dumps(data, **kwargs)

        return data

    def cmd(self, *args, **kwargs):
        data = self.main(*args, **kwargs)

        return CommandResult(data, no_format=True)



from blessings import Terminal

from jirafs import utils
from jirafs.plugin import CommandPlugin


class Command(CommandPlugin):
    """Enable/Disable or display information about installed issue plugins"""

    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def handle(self, args, folder, parser, **kwargs):
        installed_plugins = utils.get_installed_plugins()
        if args.disabled_only and args.enabled_only:
            parser.error("--disabled-only and --enabled-only are mutually exclusive.")
        if args.enable and args.enable not in installed_plugins:
            parser.error("Plugin '%s' is not installed." % args.enable)

        return self.cmd(folder, args)

    def add_arguments(self, parser):
        parser.add_argument("--verbose", action="store_true", default=False)
        parser.add_argument(
            "--enabled-only", dest="enabled_only", action="store_true", default=False
        )
        parser.add_argument(
            "--disabled-only", dest="disabled_only", action="store_true", default=False
        )
        parser.add_argument(
            "--enable",
            type=str,
        )
        parser.add_argument(
            "--disable",
            type=str,
        )
        parser.add_argument(
            "--global",
            dest="set_global",
            default=False,
            action="store_true",
        )

    def build_plugin_dict(self, enabled, available):
        all_plugins = {}
        for entrypoint_name, cls in available.items():
            all_plugins[entrypoint_name] = {
                "enabled": False,
                "class": cls,
            }
        for plugin_instance in enabled:
            entrypoint_name = plugin_instance.entrypoint_name
            all_plugins[entrypoint_name]["enabled"] = True
            all_plugins[entrypoint_name]["instance"] = plugin_instance

        return all_plugins

    def main(self, folder, args):
        t = Terminal()

        enabled_plugins = folder.load_plugins()
        available_plugins = utils.get_installed_plugins()

        if args.enable:
            if args.set_global:
                utils.set_global_config_value(
                    "plugins",
                    args.enable,
                    "enabled",
                )
            else:
                folder.set_config_value("plugins", args.enable, "enabled")
        elif args.disable:
            if args.set_global:
                utils.set_global_config_value(
                    "plugins",
                    args.disable,
                    "disabled",
                )
            else:
                folder.set_config_value("plugins", args.disable, "disabled")
        else:
            all_plugins = self.build_plugin_dict(enabled_plugins, available_plugins)

            for entrypoint_name, plugin_data in all_plugins.items():
                if plugin_data["enabled"] and args.disabled_only:
                    continue
                if not plugin_data["enabled"] and args.enabled_only:
                    continue
                if plugin_data["enabled"]:
                    color = t.bold
                else:
                    color = t.normal

                print(
                    color
                    + entrypoint_name
                    + t.normal
                    + (
                        " (Enabled)"
                        if plugin_data["enabled"]
                        else (
                            " (Disabled; enable by running `jirafs "
                            "plugins --enable=%s`)" % entrypoint_name
                        )
                    )
                )
                if args.verbose:
                    doc_string = plugin_data["class"].__doc__.strip().split("\n")
                    for line in doc_string:
                        print("     %s" % line)



from contextlib import closing
import html
import json
import mimetypes
import os
import socket
import time
import traceback
import uuid
import webbrowser
from http.server import HTTPServer
from http.server import SimpleHTTPRequestHandler

from dateutil.parser import parse

from watchdog.observers import Observer
from watchdog.events import FileSystemEventHandler

import jinja2

from jirafs import utils
from jirafs.plugin import CommandPlugin


SESSION_CONNECTED = None


class CountingEventHandler(FileSystemEventHandler):
    counter = 0

    def on_modified(self, event):
        self.counter += 1


def get_converted_markup(folder, data):
    url = folder.jira._get_url("../1.0/render")
    response = folder.jira._session.post(
        url,
        headers={"Accept": "text/html, */*; q=0.01"},
        data=json.dumps(
            {
                "issueKey": folder.ticket_number,
                "rendererType": "atlassian-wiki-renderer",
                "unrenderedMarkup": data,
            }
        ),
    )

    return response.text


class IssueRequestHandler(SimpleHTTPRequestHandler):
    folder = None
    field_data = None

    def get_rendered_template(self, template_name, context):
        loader = jinja2.FileSystemLoader(
            searchpath=os.path.join(os.path.dirname(__file__), "templates")
        )
        templates = jinja2.Environment(loader=loader)
        template = templates.get_template(template_name)
        return template.render(context)

    def get_all(self):
        lines = []

        lines.append(
            "Jump to: [#Description] | [New Comment|#NewComment] | [#Comments]"
        )
        lines.append(
            f"h1. {self.folder.issue.key}: {self.get_field_data('summary')}\n\n"
        )
        lines.append(f"h2. Description\n\n")
        description_data = self.get_field_data("description")
        if not description_data.strip():
            lines.append("_Empty_")
        else:
            lines.append(description_data)
        lines.append("\n")
        lines.append(f"h2. New Comment\n\n")
        comment_data = self.get_field_data("new_comment")
        if not comment_data:
            lines.append("_Empty_")
        else:
            lines.append(comment_data)
        lines.append("\n")
        lines.append(f"h2. Comments\n\n")
        lines.append(self.get_comments())

        return "\n".join(lines)

    def get_comments(self):
        lines = []

        for comment in self.folder.issue.fields.comment.comments:
            lines.append(
                "h3. On %s, [~%s] wrote:\n\n"
                % (
                    utils.format_date(self.folder, parse(comment.created)),
                    utils.get_comment_author_display(comment),
                )
            )
            lines.append(comment.body.replace("\r\n", "\n"))
            lines.append("\n")

        return "\n".join(lines)

    def get_field_data(self, dotpath):
        special_fields = {
            "": self.get_all,
            "new_comment": self.folder.get_new_comment,
            "comments": self.get_comments,
        }
        if dotpath in special_fields:
            data = special_fields[dotpath]()
        else:
            data = self.folder.get_field_value_by_dotpath(dotpath)

        return data

    def get_local_file_escaped_field_data(self, dotpath):
        data = self.get_field_data(dotpath)

        if not data:
            return {}, ""

        local_files = os.listdir(self.folder.path)

        referenced_files = utils.find_files_referenced_in_markup(data)

        matches_in_reverse_order = sorted(
            [
                (
                    filename,
                    match_data,
                )
                for filename, match_data in referenced_files.items()
            ],
            key=lambda match: -1 * match[1][2],
        )
        placeholders = {}
        for filename, (full, start, end) in matches_in_reverse_order:
            if filename not in local_files:
                continue

            id = uuid.uuid4()
            placeholder = f"JIRAFS-PLACEHOLDER:{id}"
            placeholders[placeholder] = (filename, full)
            data = "".join([data[:start], placeholder, data[end:]])

        return placeholders, data

    def get_issue_title(self, html_title):
        return f"[{self.folder.issue.key}]: {html_title}"

    def replace_placeholders(self, placeholders, data):
        for placeholder, (filename, full) in placeholders.items():
            if full.startswith("!"):
                data = data.replace(placeholder, f'<img src="files/{filename}" />')
            elif full.startswith("[^"):
                data = data.replace(
                    placeholder, f'<a href="files/{filename}">{filename}</a>'
                )

        return data

    def serve_preview_content(self, dotpath):
        content_type = "text/html"
        placeholders, data = self.get_local_file_escaped_field_data(dotpath)

        html_title = dotpath
        if not html_title:
            html_title = self.get_field_data("summary")

        if isinstance(data, str):
            response = self.get_rendered_template(
                "base.html",
                {
                    "content": self.replace_placeholders(
                        placeholders, get_converted_markup(self.folder, data)
                    ),
                    "title": self.get_issue_title(html_title),
                },
            )
        else:
            response = json.dumps(data)
            content_type = "application/json"
        self.send_response(200)
        self.send_header("Content-type", content_type)
        self.send_header("Content-length", len(response))
        self.end_headers()
        self.wfile.write(response.encode("utf-8"))

    def serve_file(self, path):
        if path not in os.listdir(self.folder.path):
            self.send_response(404)
            self.send_header("Content-length", 0)
            self.end_headers()
            return

        with open(os.path.join(self.folder.path, path), "rb") as inf:
            self.send_response(200)
            inf.seek(0, 2)
            content_type = mimetypes.guess_type(path)[0] or "application/octet-stream"
            self.send_header("Content-type", content_type)
            self.send_header("Content-length", inf.tell())
            self.end_headers()
            inf.seek(0)
            self.wfile.write(inf.read())

    def send_eventsource_message(self, message):
        self.wfile.write(str(len(message.encode("utf-8")) + 2).encode("utf-8"))
        self.wfile.write("\r\n".encode("utf-8"))
        self.wfile.write(message.encode("utf-8"))
        self.wfile.write("\r\n".encode("utf-8"))

    def serve_eventsource(self):
        event_handler = CountingEventHandler()
        observer = Observer()
        observer.schedule(event_handler, path=self.folder.path, recursive=True)
        observer.start()

        self.send_response(200)
        self.send_header("Transfer-encoding", "chunked")
        self.send_header("Content-Type", "text/event-stream")
        self.end_headers()

        while True:
            self.send_eventsource_message(
                "event: counter\r\ndata: %s\r\n" % event_handler.counter
            )
            time.sleep(0.5)

    def do_DELETE(self):
        global SESSION_CONNECTED

        if self.path == "/eventsource/":
            SESSION_CONNECTED = False
            self.send_response(200)
        else:
            self.send_response(404)
        self.end_headers()

    def do_GET(self):
        global SESSION_COUNTER

        self.folder.clear_cache()

        try:
            if self.path.startswith("/files/"):
                self.serve_file(self.path[7:])
            elif self.path == "/eventsource/":
                SESSION_COUNTER = True

                self.serve_eventsource()
            else:
                self.serve_preview_content(self.path[1:].replace("/", "."))
        except BrokenPipeError:
            pass
        except Exception as e:
            self.send_response(500)
            response = self.get_rendered_template(
                "traceback.html",
                {
                    "content": html.escape(traceback.format_exc()),
                    "title": f"Error: {e}",
                },
            )
            self.send_header("Content-type", "text/html")
            self.send_header("Content-length", len(response))
            self.end_headers()
            self.wfile.write(response.encode("utf-8"))


class Command(CommandPlugin):
    """Preview your Jira wiki markup"""

    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def add_arguments(self, parser):
        parser.add_argument(
            "--port",
            "-p",
            help=(
                "Start a webserver on this port; defaults to asking "
                "the operating system for any available port."
            ),
            type=int,
            default=0,
        )
        parser.add_argument(
            "--no-browser",
            "-n",
            action="store_true",
            default=False,
            help=("Do not open a webbrowser to the created webserver."),
        )
        parser.add_argument(
            "--serve-forever",
            "-f",
            action="store_true",
            default=False,
            help=(
                "Do not automatically terminate preview session "
                "when user navigates away from preview URL."
            ),
        )
        parser.add_argument("field_name", nargs="?")

    def handle(self, args, folder, **kwargs):
        return self.cmd(
            folder,
            args.field_name or "",
            port=args.port,
            open_browser=not args.no_browser,
            serve_forever=args.serve_forever,
        )

    def continue_serving(self, serve_forever=True):
        if serve_forever:
            return True

        if SESSION_CONNECTED is None or SESSION_CONNECTED:
            return True

        return False

    def main(
        self,
        folder,
        field_name,
        port=0,
        open_browser=True,
        serve_forever=True,
        **kwargs,
    ):
        if os.path.isfile(field_name) and field_name.endswith(".jira"):
            field_name = field_name.split(".")[0]

        with closing(socket.socket(socket.AF_INET, socket.SOCK_STREAM)) as s:
            s.bind(("", port))
            s.setsockopt(socket.SOL_SOCKET, socket.SO_REUSEADDR, 1)
            port = s.getsockname()[1]
            path = field_name.replace(".", "/")

            IssueRequestHandler.folder = folder
            IssueRequestHandler.get_converted_markup = get_converted_markup

            server = HTTPServer(("", port), IssueRequestHandler)
            server.timeout = 0.1
            print(f"Serving from http://127.0.0.1:{port}")
            print("Press <Ctrl+C> to Exit")

            if open_browser:
                webbrowser.open(f"http://127.0.0.1:{port}/{path}")

            try:
                while self.continue_serving(serve_forever):
                    server.handle_request()
            except KeyboardInterrupt:
                pass



from jirafs.plugin import CommandPlugin
from jirafs.utils import run_command_method_with_kwargs


class Command(CommandPlugin):
    """Commit current changes, push changes to Jira, and pull changes"""

    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"

    def add_arguments(self, parser):
        parser.add_argument("-m", "--message", dest="message", default="Untitled")

    def handle(self, args, folder, **kwargs):
        return self.cmd(folder, args.message)

    def main(self, folder, message):
        commit_result = run_command_method_with_kwargs(
            "commit", folder=folder, message=message
        )
        push_result = run_command_method_with_kwargs("push", folder=folder)

        return commit_result, push_result



from jirafs.plugin import CommandPlugin
from jirafs import __version__


class Command(CommandPlugin):
    """Print the current version number to the console"""

    MIN_VERSION = "2.0.0"
    MAX_VERSION = "3.0.0"
    AUTOMATICALLY_INSTANTIATE_FOLDER = False

    def handle(self, args, *vargs, **kwargs):
        print(f"Jirafs version {__version__}")
